"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getColor = exports.getDisplayName = exports.aggregate = exports.getSortedSamples = exports.createSortEventsForGeneComparator = exports.createSamplesMap = exports.isMutation = exports.getEventRatiosPerGene = exports.getSortedGenes = exports.getGeneNames = exports.SupportedEvents = void 0;

var _lodash = _interopRequireDefault(require("lodash"));

var _PrecomputedComparator = _interopRequireDefault(require("./PrecomputedComparator"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

var MutationEventTypes = ['INFRAME', 'TRUNC', 'MISSENSE'];
var SupportedEvents = {
  // Mutations
  MISSENSE: {
    colorHTML: '#008000',
    displayName: 'Missense mutation'
  },
  INFRAME: {
    colorHTML: '#993404',
    displayName: 'Inframe mutation'
  },
  TRUNC: {
    colorHTML: '#000000',
    displayName: 'Truncation mutation'
  },
  // Fusion
  FUSION: {
    colorHTML: '#8b00c9',
    displayName: 'Fusion'
  },
  // Copy number alterations
  AMP: {
    colorHTML: '#ff0000',
    displayName: 'Amplification'
  },
  GAIN: {
    colorHTML: '#ffb6c1',
    displayName: 'Gain'
  },
  HETLOSS: {
    colorHTML: '#8fd8d8',
    displayName: 'Shallow deletion'
  },
  HOMDEL: {
    colorHTML: '#0000ff',
    displayName: 'Deep deletion'
  },
  // mRNA expressions
  UP: {
    colorHTML: '#ff9999',
    displayName: 'mRNA Upregulation'
  },
  DOWN: {
    colorHTML: '#6699cc',
    displayName: 'mRNA Downregulation'
  }
}; // Describes the order of importance for CNA events.

exports.SupportedEvents = SupportedEvents;
var AlterationsOrder = {
  AMP: 0,
  GAIN: 2,
  HETLOSS: 3,
  HOMDEL: 1,
  undefined: 4
}; // Describes the order of importance for mutation events.

var MutationsOrder = {
  INFRAME: 1,
  MISSENSE: 3,
  TRUNC: 0,
  undefined: 4
}; // Describes the order of importance for mRNA expression events.

var ExpressionsOrder = {
  UP: 0,
  DOWN: 1,
  undefined: 2
}; // Retrieves the gene names in a set of events.

var getGeneNames = function getGeneNames(events) {
  return events.map(function (e) {
    return e.gene;
  }).filter(function (gene) {
    return gene !== null;
  });
}; // Returns the set of genes (unique) reversed to display on the Y axis.


exports.getGeneNames = getGeneNames;

var getSortedGenes = function getSortedGenes(events) {
  return _toConsumableArray(new Set(getGeneNames(events))).reverse();
}; // Returns a hash map with the percentage of events (value) per gene (key).


exports.getSortedGenes = getSortedGenes;

var getEventRatiosPerGene = function getEventRatiosPerGene(events, nbSamples) {
  var map = events.reduce(function (acc, event) {
    if (event.type) {
      if (acc[event.gene]) {
        acc[event.gene] += 1;
      } else {
        acc[event.gene] = 1;
      }
    }

    return acc;
  }, {});
  Object.keys(map).forEach(function (gene) {
    map[gene] = Math.floor(map[gene] / nbSamples * 100);
  });
  return map;
}; // Returns true if an event is a mutation, false otherwise.


exports.getEventRatiosPerGene = getEventRatiosPerGene;

var isMutation = function isMutation(event) {
  return MutationEventTypes.includes(event.type);
}; // Returns a comparator result value given an integer that may not be -1, 0 or
// 1 (which are the only allowed sorting return values).


exports.isMutation = isMutation;

var sign = function sign(x) {
  if (x > 0) {
    return 1;
  } else if (x < 0) {
    return -1;
  }

  return 0;
}; // Returns a comparator for the samples (matrix column), combining all the
// precomputed comparators created for each gene (i.e. matrix row).


var samplesComparator = function samplesComparator(genes, samplesToIndex, perGeneComparators) {
  return function (s1, s2) {
    var result = 0;
    var absoluteResult = 0;

    for (var i = 0; i < genes.length; i += 1) {
      var nextResult = perGeneComparators[i].compare(s1, s2);
      var nextAbsoluteResult = Math.abs(nextResult);

      if (nextAbsoluteResult > absoluteResult) {
        result = nextResult;
        absoluteResult = nextAbsoluteResult;
      }

      if (absoluteResult === 1) {
        break;
      }
    }

    if (result === 0) {
      result = samplesToIndex[s1] < samplesToIndex[s2] ? -1 : 1;
    }

    return result > 0 ? 1 : -1;
  };
}; // Returns a comparator to sort the samples given a gene .


var sortEventsForGene = function sortEventsForGene(s1, s2, gene, samplesMap) {
  var d1 = samplesMap[s1][gene] || {};
  var d2 = samplesMap[s2][gene] || {};

  if (d1.FUSION && !d2.FUSION) {
    return -1;
  } else if (!d1.FUSION && d2.FUSION) {
    return 1;
  }

  var cna = sign(AlterationsOrder[d1.CNA] - AlterationsOrder[d2.CNA]);

  if (cna !== 0) {
    return cna;
  }

  var mut = sign(MutationsOrder[d1.MUT] - MutationsOrder[d2.MUT]);

  if (mut !== 0) {
    return mut;
  }

  var exp = sign(ExpressionsOrder[d1.EXP] - ExpressionsOrder[d2.EXP]);

  if (exp !== 0) {
    return exp;
  }

  return 0;
}; // Returns a map to gather information on each sample, per gene, per event.


var createSamplesMap = function createSamplesMap(events) {
  var samplesMap = {};
  events.forEach(function (e) {
    var s = samplesMap[e.sample] || {};
    var v = s[e.gene] || {};

    if (isMutation(e)) {
      v.MUT = e.type;
    } else {
      v[e.type] = e.alteration;
    }

    samplesMap[e.sample] = Object.assign({}, samplesMap[e.sample], _defineProperty({}, e.gene, v));
  });
  return samplesMap;
}; // Helper function to create a comparator for each gene.


exports.createSamplesMap = createSamplesMap;

var createSortEventsForGeneComparator = function createSortEventsForGeneComparator(gene, map) {
  return function (s1, s2) {
    return sortEventsForGene(s1, s2, gene, map);
  };
}; // Returns the list of samples sorted with mutual exclusion. The sorting
// algorithm is similar to the one used on cBioPortal and takes both the rows
// (genes) and columns (samples) into account. We returns the sorted set of
// samples to display on X axix.


exports.createSortEventsForGeneComparator = createSortEventsForGeneComparator;

var getSortedSamples = function getSortedSamples(events) {
  // Get a map with samples sorted by gene and events.
  var samplesMap = createSamplesMap(events); // Get a unique list of genes, sorted by the natural order in the events.

  var genes = _toConsumableArray(new Set(getGeneNames(events))); // Sort the samples alphabetically.


  var samples = _toConsumableArray(new Set(events.map(function (e) {
    return e.sample;
  }))).sort(); // Build one comparator per gene.


  var perGeneComparators = [];
  genes.forEach(function (gene) {
    perGeneComparators.push( // This actually sorts the samples, but for each gene only.
    new _PrecomputedComparator.default(_toConsumableArray(samples), createSortEventsForGeneComparator(gene, samplesMap)));
  }); // Create a map with the current order of the samples.

  var samplesToIndex = {};
  samples.forEach(function (s, i) {
    samplesToIndex[s] = i;
  }); // Finally, sort the samples taking into account both the columns and rows.

  var sortedSamples = _toConsumableArray(samples);

  sortedSamples.sort(samplesComparator(genes, samplesToIndex, perGeneComparators));
  return sortedSamples;
}; // Returns the events aggregated by type (if mutation) or alteration.


exports.getSortedSamples = getSortedSamples;

var aggregate = function aggregate(events) {
  var out = {};
  events.forEach(function (e) {
    if (!e.type || e.type === 'NONE') {
      return;
    }

    var k = isMutation(e) ? e.type : e.alteration;
    var v = out[k] || {
      type: e.type,
      alteration: e.alteration,
      events: []
    };
    v.events.push(e);
    out[k] = v;
  });
  return out;
}; // Returns the display name of an event.


exports.aggregate = aggregate;

var getDisplayName = function getDisplayName(event) {
  var eventName = isMutation(event) ? event.type : event.alteration;
  return SupportedEvents[eventName].displayName;
}; // Returns the color of an event.


exports.getDisplayName = getDisplayName;

var getColor = function getColor(event, colorscale) {
  var eventName = isMutation(event) ? event.type : event.alteration;
  var color;

  if (colorscale && _lodash.default.isObject(colorscale)) {
    color = colorscale[eventName]; // Revert back to default scale if not found

    if (!color) {
      color = SupportedEvents[eventName].colorHTML;
    }
  } else {
    color = SupportedEvents[eventName].colorHTML;
  }

  return color;
};

exports.getColor = getColor;